package com.restlet.client.commons;

import static com.restlet.client.utils.Objects.requireNonNull;
import static com.restlet.client.utils.StringUtils.format;

public abstract class AppConstants {

    public enum ExternalLink {
        AddChromeReview(getChromeExtensionReviewsUrl(), "writing a review", false),
        ClientExtension(getChromeExtensionUrl(), getClientModuleName(), false),
        ClientDocumentation(getClientDocumentationUrl(), "Documentation", true),
        ClientMavenPluginDocumentation(getMavenPluginDocumentationUrl(), "CI documentation", true),
        ClientPage(getClientPageUrl(), getClientModuleName(), true),
        ClientApp(getClientAppUrl(), getClientModuleName(), false),
        CloudPage(getCloudPageUrl(), getCloudModuleName(), true),
        CloudApp(getCloudAppUrl(), getCloudModuleName(), false),
        StudioPage(getStudioPageUrl(), getStudioModuleName(), true),
        StudioApp(getStudioAppUrl(), getStudioModuleName(), false);

        public final String label;
        public final String baseLink;
        public final boolean shouldAddUtm;

        ExternalLink(final String baseLink, final String label, final boolean shouldAddUtm) {
            this.baseLink = baseLink;
            this.label = label;
            this.shouldAddUtm = shouldAddUtm;
        }

        public String getUrl(final String subLink) {
            requireNonNull(subLink);
            return format("%s%s%s",
                    baseLink,
                    subLink,
                    shouldAddUtm ? "?utm_source=DHC" : "");
        }

        public String getUrl() {
            return format("%s%s",
                    baseLink,
                    shouldAddUtm ? "?utm_source=DHC" : "");
        }
    }

    public static String getClientName() { return "client"; }

    public static String getProviderDomain() {
        return "client.restlet.com";
    }

    public static String getTwitterId() {
        return "restlet";
    }

    public static String getSupportEmail() {
        return "support@restlet.com";
    }

    public static String getTwitterUrl() {
        return "https://twitter.com/restlet";
    }

    public static String getClientModuleName() {
        return "Restlet Client";
    }

    static String getClientDocumentationUrl() {
        return "https://restlet.com/documentation/client/user-guide";
    }

    public static String getMavenPluginDocumentationUrl() {
        return getClientDocumentationUrl() + "/run-tests/automate-tests/test-reports-maven";
    }

    public static String getMavenPluginNexusUrl() {
        return "http://maven.restlet.com/com/restlet/client/maven-plugin";
    }

    static String getClientPageUrl() { return "https://restlet.com/modules/client"; }

    static String getClientAppUrl() {
        return "https://client.restlet.com";
    }

    static String getStudioModuleName() {
        return "Restlet Studio";
    }

    static String getStudioPageUrl() { return "https://restlet.com/modules/studio"; }

    static String getStudioAppUrl() {
        return "https://studio.restlet.com";
    }

    static String getCloudModuleName() {
        return "Restlet Cloud";
    }

    static String getCloudPageUrl() { return "https://restlet.com/modules/cloud"; }

    static String getCloudAppUrl() {
        return "https://cloud.restlet.com";
    }

    public static String getChromeExtensionUrl() {
        return "https://chrome.google.com/webstore/detail/restlet-client/aejoelaoggembcahagimdiliamlcdmfm";
    }

    public static String getChromeExtensionReviewsUrl() { return getChromeExtensionUrl() + "/reviews"; }

    public static String getSystinetName() {
        return "HP Systinet";
    }

    public static String getSwaggerName() {
        return "Swagger";
    }

    public static String getHarName() {
        return "HAR";
    }

    public static String getMixpanelInitialReferrerDomainForRestletClientProduct() {
        return "chrome.google.com";
    }

    public static String getMixpanelInitialReferrerDomainForOtherPlatformProducts() {
        return "restlet.com";
    }

}
