package com.restlet.client.model;

import com.restlet.client.function.Function;
import com.restlet.client.function.Predicate;
import com.restlet.client.utils.Sequence;

import java.util.Set;

public enum ImportPolicy {
    Overwrite("Overwrite", true, true),
    Update("Update", true, false),
    Preserve("Preserve", true, true),
    Sharing("Sharing", false, false),
    Synchronization("Synchronization", false, false);

    public final String label;
    public final boolean isFileImportPolicy;
    public final boolean isBridgeImportPolicy;

    ImportPolicy(final String label, final boolean isFileImportPolicy, final boolean isBridgeImportPolicy) {
        this.label = label;
        this.isFileImportPolicy = isFileImportPolicy;
        this.isBridgeImportPolicy = isBridgeImportPolicy;
    }

    public static Set<ImportPolicy> getFileImportPolicies() {
        return Sequence.of(values())
                .filter(isFileImportPolicyFilter())
                .toSet();
    }

    private static Predicate<ImportPolicy> isFileImportPolicyFilter() {
        return new Predicate<ImportPolicy>() {
            @Override
            public boolean test(final ImportPolicy importPolicy) {
                return importPolicy.isFileImportPolicy;
            }
        };
    }

    public Function<EntityTreeNode, String> identifierRetriever() {
        return new Function<EntityTreeNode, String>() {
            @Override
            public String apply(final EntityTreeNode treeNode) {
                return isFileImportPolicy
                        ? treeNode.entity.getName()
                        : treeNode.entity.getId();
            }
        };
    }

    public Function<String, EntityTreeNode> childRetriever(final EntityTreeNode parent) {
        return new Function<String, EntityTreeNode>() {
            @Override
            public EntityTreeNode apply(final String identifier) {
                return isFileImportPolicy
                        ? parent.retrieveChildByName(identifier)
                        : parent.retrieveChildById(identifier);
            }
        };
    }
}
