package com.restlet.client.model.impl;

import com.restlet.client.model.EntityTo;
import com.restlet.client.model.ProjectTo;
import com.restlet.client.model.ServiceTo;
import com.restlet.client.model.environment.EnvironmentTo;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.platform.json.JsonArray;
import com.restlet.client.platform.json.JsonEngine;
import com.restlet.client.platform.json.JsonException;
import com.restlet.client.platform.json.JsonObject;
import com.restlet.client.platform.json.JsonValue;
import com.restlet.client.tests.ScenarioTo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.restlet.client.utils.StringUtils.isBlank;

public class ImportParserImpl {

    private final JsonEngine jsonEngine;

    public ImportParserImpl(final JsonEngine jsonEngine) {
        this.jsonEngine = jsonEngine;
    }

    public String findFileVersion(final String fileContent) {
        try {
            return jsonEngine.fromJson(fileContent).isObject()
                    .get("front-version").isString().stringValue();
        } catch (final Throwable e) {
            return "1.3.5 or earlier";
        }
    }

    public Map<String, EntityTo> parse(final String o) throws JsonException {

        final JsonValue repository = jsonEngine.fromJson(o);
        if (repository == null) {
            return null;
        }

        final JsonObject repo = repository.isObject();

        if (repo == null) {
            return null;
        }

        if (!repo.containsKey("version") || repo.get("version") == null || repo.get("version").isNumber() == null) {
            return null;
        }

        final int version = (int) repo.get("version").isNumber().doubleValue();

        if (version < 3) {
            //TODO throw an exception
            return null;
        }

        if (repo.get("nodes") == null) {
            return null;
        }

        final JsonArray nodes = repo.get("nodes").isArray();

        if (nodes == null) {
            return null;
        }

        final List<JsonObject> repositoryNodes = new ArrayList<>();

        for (int i = 0; i < nodes.size(); i++) {

            final JsonObject obj = nodes.get(i).isObject();
            if (obj != null) {
                repositoryNodes.add(obj);
            }
        }

        final Map<String, EntityTo> entities = new HashMap<>();

        for (final JsonObject repositoryNode : repositoryNodes) {

            final EntityTo entity = getEntity(repositoryNode);
            if (entity != null) {
                entities.put(entity.getId(), entity);
            }
        }

        for (final EntityTo entityTo : entities.values()) {

            final EntityTo parent = entityTo.getParent();

            if (parent == null) {
                continue;
            }

            final String parentId = parent.getId();
            if (!isBlank(parentId)) {
                entityTo.setParent(entities.get(parentId));
            }
        }

        return entities;
    }

    private EntityTo getEntity(final JsonObject object) throws JsonException {

        final String type = object.get("type") != null && object.get("type").isString() != null ? object.get("type").isString().stringValue() : null;
        if (isBlank(type)) {
            return null;
        }

        final EntityTo.Type entityType = EntityTo.Type.valueOf(type);
        EntityTo entity = null;

        switch (entityType) {
            case Context:
                entity = jsonEngine.fromJson(jsonEngine.toJson(object), EnvironmentTo.class);
                break;

            case Project:
                entity = jsonEngine.fromJson(jsonEngine.toJson(object), ProjectTo.class);
                break;

            case Request:
                entity = jsonEngine.fromJson(jsonEngine.toJson(object), HttpRequestTo.class);
                break;

            case Service:
                entity = jsonEngine.fromJson(jsonEngine.toJson(object), ServiceTo.class);
                break;

            case Scenario:
                entity = jsonEngine.fromJson(jsonEngine.toJson(object), ScenarioTo.class);
                break;

        }

        if (entity == null) {
            return null;
        }

        final String parentId = object.get("parentId") != null && object.get("parentId").isString() != null ? object.get("parentId").isString().stringValue() : null;

        if (isBlank(parentId)) {
            entity.setParent(null);
        } else {
            final ProjectTo parent = new ProjectToImpl();
            parent.setId(parentId);
            entity.setParent(parent);
        }

        return entity;
    }

}
