package com.restlet.client.platform.blob;

import com.restlet.client.async.Deferred;
import com.restlet.client.async.Promise;
import com.restlet.client.async.Promises;
import com.restlet.client.async.impl.DeferredImpl;

public interface BlobReader {

    void readAsText(Blob blob, Handler<String> handler);

    Promise<String> readAsText(Blob blob);

    interface Handler<T> {

        void onLoaded(T o);

        void onError(Throwable o);
    }

    abstract class AbstractBlobReader implements BlobReader {

        @Override
        public Promise<String> readAsText(final Blob blob) {
            if (blob == null || blob.isEmpty()) {
                return Promises.of();
            }

            final Deferred<String> deferred = new DeferredImpl<>();

            readAsText(blob, new Handler<String>() {
                @Override
                public void onLoaded(final String text) {
                    deferred.resolve(text);
                }

                @Override
                public void onError(final Throwable error) {
                    deferred.reject(error);
                }
            });

            return deferred.promise();
        }
    }
}
