package com.restlet.client.platform.json;

import com.restlet.client.utils.Objects;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

public class JsonObjectImpl implements JsonObject {

    private final Map<String, Object> object;
    private final JsonEngine jsonEngine;

    public JsonObjectImpl(JsonEngine jsonEngine) {
        this(new HashMap<String, Object>(), jsonEngine);
    }

    public JsonObjectImpl(Map<String, Object> object, JsonEngine jsonEngine) {
        this.object = object;
        this.jsonEngine = jsonEngine;
    }

    protected static Object unwrap(JsonValue value) {
        if (value == null) {
            return null;
        }
        if (value instanceof JsonObjectImpl) {
            return ((JsonObjectImpl) value).map();
        }
        if (value instanceof JsonString) {
            return ((JsonString) value).stringValue();
        }
        if (value instanceof JsonNumber) {
            return ((JsonNumber) value).doubleValue();
        }
        if (value instanceof JsonArrayImpl) {
            return ((JsonArrayImpl) value).collection();
        }
        if (value instanceof JsonBoolean) {
            return ((JsonBoolean) value).booleanValue();
        }
        if (value instanceof JsonNull) {
            return null;
        }
        return null;
    }

    protected static JsonValue wrap(Object value, JsonEngine jsonEngine) {
        if (value != null) {
            if (value instanceof Map) {
                return new JsonObjectImpl((Map) value, jsonEngine);
            }
            if (value instanceof String) {
                return new JsonStringImpl((String) value);
            }
            if (value instanceof Integer) {
                return new JsonNumberImpl((Integer) value);
            }
            if (value instanceof Long) {
                return new JsonNumberImpl((Long) value);
            }
            if (value instanceof Float) {
                return new JsonNumberImpl((Float) value);
            }
            if (value instanceof Double) {
                return new JsonNumberImpl((Double) value);
            }
            if (value instanceof Collection) {
                return new JsonArrayImpl((Collection) value, jsonEngine);
            }
            if (value instanceof Boolean) {
                return new JsonBooleanImpl((boolean) value);
            }
        }

        return null;
    }

    @Override
    public JsonObject isObject() {
        return this;
    }

    @Override
    public JsonArray isArray() {
        return null;
    }

    @Override
    public JsonNull isNull() {
        return null;
    }

    @Override
    public JsonString isString() {
        return null;
    }

    @Override
    public JsonBoolean isBoolean() {
        return null;
    }

    @Override
    public JsonNumber isNumber() {
        return null;
    }

    @Override
    public JsonValue get(String path) {
        return wrap(object.get(path), this.jsonEngine);
    }

    @Override
    public Set<String> keySet() {
        return object.keySet();
    }

    @Override
    public boolean containsKey(String key) {
        return object.containsKey(key);
    }

    @Override
    public JsonObject put(String name, JsonValue value) {
        object.put(name, unwrap(value));
        return this;
    }

    public Map<String, Object> map() {
        return object;
    }

    @Override
    public Integer size() {
        return object.size();
    }

    @Override
    public String toString() {
        if (object == null) {
            return "null";
        }
        return jsonEngine.toJson(this);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        return Objects.equals(object, ((JsonObjectImpl) o).object);
    }

    @Override
    public int hashCode() {
        return object.hashCode();
    }
}
