package com.restlet.client.tests.impl;

import com.restlet.client.function.Consumer;
import com.restlet.client.function.Function;
import com.restlet.client.function.Predicate;
import com.restlet.client.model.EntityTo;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.net.http.request.impl.EffectiveHttpRequestTo;
import com.restlet.client.net.request.RetryConfiguration;
import com.restlet.client.tests.ContainerTestResult;
import com.restlet.client.tests.SingleRequestTestResult;
import com.restlet.client.tests.SingleRequestTryTestResult;
import com.restlet.client.tests.TestResult;
import com.restlet.client.tests.impl.ExecutionStateStore.RunType;
import com.restlet.client.utils.Iterables;
import com.restlet.client.utils.Maybe;
import com.restlet.client.utils.Objects;
import com.restlet.client.utils.Sequence;

import java.util.ArrayList;
import java.util.List;

import static com.restlet.client.tests.TestResult.State.InProgress;
import static com.restlet.client.tests.TestResult.State.Skipped;
import static com.restlet.client.utils.EntityUtils.generateDottedName;
import static com.restlet.client.utils.FunctionalUtils.ifNotNull;

public class SingleRequestTestResultImpl implements SingleRequestTestResult {

    private State state;

    private String message;

    private EffectiveHttpRequestTo effectiveHttpRequestTo;

    private String name;

    private String fullName;

    private boolean dynamicRequest;

    private boolean localRequest;

    private int currentTryBaseOneIndex;

    private ContainerTestResult parent;

    private HttpRequestTo entity;

    private long elapsedTime = 0L;

    private boolean requestInvalid;

    private final List<SingleRequestTryTestResult> triesResults = new ArrayList<>();

    public SingleRequestTestResultImpl() {
        this.state = InProgress;
    }

    @Override
    public State getState() {
        return state;
    }

    @Override
    public SingleRequestTestResult setState(final State state) {
        this.state = state;
        return this;
    }

    @Override
    public String getMessage() {
        return message;
    }

    @Override
    public SingleRequestTestResultImpl setMessage(final String message) {
        this.message = message;
        return this;
    }

    @Override
    public EffectiveHttpRequestTo getEffectiveRequest() {
        return effectiveHttpRequestTo;
    }

    @Override
    public SingleRequestTestResultImpl setEffectiveRequest(final EffectiveHttpRequestTo effectiveHttpRequestTo) {
        this.effectiveHttpRequestTo = effectiveHttpRequestTo;
        return this;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public SingleRequestTestResult setName(final String name) {
        this.name = name;
        return this;
    }

    @Override
    public String getFullName() {
        return fullName;
    }

    @Override
    public SingleRequestTestResult setFullName(final String fullName) {
        this.fullName = fullName;
        return this;
    }

    @Override
    public boolean isDynamicRequest() {
        return dynamicRequest;
    }

    @Override
    public boolean isEnded() {
        return getState().isTerminal;
    }

    @Override
    public SingleRequestTestResult setDynamicRequest(final boolean dynamicRequest) {
        this.dynamicRequest = dynamicRequest;
        return this;
    }

    @Override
    public boolean isLocalRequest() {
        return localRequest;
    }

    @Override
    public SingleRequestTestResult setLocalRequest(final boolean localRequest) {
        this.localRequest = localRequest;
        return this;
    }

    @Override
    public SingleRequestTestResult refreshParentState(final RunType runType, final boolean fireEvent) {
        if (parent != null) {
            parent.refreshState(runType, fireEvent);
        }

        return this;
    }

    @Override
    public ContainerTestResult getParent() {
        return parent;
    }

    @Override
    public SingleRequestTestResult setParent(final ContainerTestResult parent) {
        this.parent = parent;
        return this;
    }

    @Override
    public HttpRequestTo getEntity() {
        return entity;
    }

    @Override
    public SingleRequestTestResult setEntity(final HttpRequestTo entity) {
        this.entity = entity;
        return this;
    }

    @Override
    public long getElapsedTime() {
        return elapsedTime;
    }

    @Override
    public SingleRequestTestResult setElapsedTime(final long elapsedTime) {
        this.elapsedTime = elapsedTime;
        return this;
    }

    @Override
    public List<SingleRequestTryTestResult> getTriesResults() {
        return triesResults;
    }

    @Override
    public SingleRequestTestResult addTryResult(final SingleRequestTryTestResult tryResult) {
        this.triesResults.add(tryResult);
        tryResult.setParent(this);
        return this;
    }

    @Override
    public Maybe<SingleRequestTryTestResult> getLastNonSkippedTry() {
        final List<SingleRequestTryTestResult> nonSkippedTries = Sequence.of(triesResults)
                .omit(new Predicate<SingleRequestTryTestResult>() {
                    @Override
                    public boolean test(final SingleRequestTryTestResult tryResult) {
                        return tryResult.getState() == Skipped;
                    }
                })
                .toList();

        return nonSkippedTries.isEmpty() ? Maybe.<SingleRequestTryTestResult>empty() : Maybe.ofNullable(Iterables.getLast(nonSkippedTries));
    }

    @Override
    public boolean isRetryEnabled() {
        return getEntity() != null && ifNotNull(getEntity().getRetryConfiguration(), new Function<RetryConfiguration, Boolean>() {
            @Override
            public Boolean apply(final RetryConfiguration retryConfiguration) {
                return retryConfiguration.isEnabled();
            }
        }, false);
    }

    @Override
    public int getCurrentTryBaseOneRetryIndex() {
        return currentTryBaseOneIndex;
    }

    @Override
    public SingleRequestTestResult setCurrentTryBaseOneRetryIndex(final int currentTryBaseOneIndex) {
        this.currentTryBaseOneIndex = currentTryBaseOneIndex;
        return this;
    }

    @Override
    public SingleRequestTestResult computeNameAndFullName() {
        this.name = entity.getName();
        this.fullName = generateDottedName(entity);
        return this;
    }

    @Override
    public SingleRequestTestResult walk(final Consumer<TestResult> visitor) {
        visitor.consume(this);
        return this;
    }

    @Override
    public boolean isRequestInvalid() {
        return requestInvalid;
    }

    @Override
    public SingleRequestTestResult setRequestInvalid(final boolean isRequestInvalid) {
        this.requestInvalid = isRequestInvalid;
        return this;
    }

    @Override
    public boolean matches(final EntityTo entity) {
        return matches(entity.getId());
    }

    @Override
    public boolean matches(final String entityId) {
        return Objects.equals(entityId, getEntity().getId());
    }

}
