package com.restlet.client.utils;

import com.restlet.client.function.Function;
import com.restlet.client.model.EntityTo;
import com.restlet.client.model.EntityTreeNode;

import java.util.HashMap;
import java.util.Map;

public abstract class EntityCloner {

    public static EntityCloner ENTITY_CLONER;

    public abstract <T extends EntityTo> T clone(T entity, final Map<String, EntityTo> cloneDictionary);

    /**
     * @deprecated Use clone instead
     */
    @Deprecated
    public abstract <T extends EntityTo> T cloneKeepParentsAsIs(T entity);

    public EntityTreeNode cloneEntityTreeNode(final EntityTreeNode nodeToClone) {
        return cloneEntityTreeNode(nodeToClone, new HashMap<String, EntityTo>());
    }

    private EntityTreeNode cloneEntityTreeNode(final EntityTreeNode nodeToClone, final Map<String, EntityTo> cloneDictionary) {
        final EntityTo clone = clone(nodeToClone.entity, cloneDictionary);
        cloneDictionary.put(clone.getId(), clone);
        final EntityTreeNode clonedTreeNode = new EntityTreeNode(clone, nodeToClone.updateType);
        clonedTreeNode.addChildren(
                Sequence.of(nodeToClone.children)
                        .map(new Function<EntityTreeNode, EntityTreeNode>() {
                            @Override
                            public EntityTreeNode apply(final EntityTreeNode nodeToClone1) {
                                return EntityCloner.this.cloneEntityTreeNode(nodeToClone1, cloneDictionary);
                            }
                        })
                        .toList()
        );
        return clonedTreeNode;
    }
}
