package com.restlet.client.utils;

import com.restlet.client.function.Supplier;

import java.util.HashMap;
import java.util.Map;

public class MapBuilder<K, V> {

    private final Map<K, V> local;

    private MapBuilder(Supplier<Map<K, V>> constructor) {
        local = constructor.get();
    }

    public static <K, V> MapBuilder<K, V> builder() {
        return new MapBuilder<>(new Supplier<Map<K, V>>() {
            @Override
            public Map<K, V> get() {
                return new HashMap<>();
            }
        });
    }

    public static <K, V> MapBuilder<K, V> builder(Supplier<Map<K, V>> constructor) {
        return new MapBuilder<>(constructor);
    }

    public MapBuilder<K, V> put(K key, V value) {
        local.put(key, value);
        return this;
    }

    public MapBuilder<K, V> putAll(Map<? extends K, ? extends V> map) {
        local.putAll(map);
        return this;
    }

    public Map<K, V> build() {
        return local;
    }

}
